// API - HPDI32 - Any OS
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/dev_io_dma.c $
// $Rev: 23674 $
// $Date: 2013-10-31 16:25:34 -0500 (Thu, 31 Oct 2013) $

#include <string.h>

#include "dev_io.h"

#include "dev_access.h"
#include "dev_irq.h"
#include "dev_reg.h"

#include "gsc_os_time.h"



/* prototypes	*************************************************************/

static	U32	_dma_read_wait(gsc_io_t* gio, U32 timeout_ms);
static	U32	_dma_rx_available(gsc_io_t* gio);
static	U32	_dma_tx_available(gsc_io_t* gio);
static	U32	_dma_write_wait(gsc_io_t* gio, U32 timeout_ms);
static	U32	_dmdma_rx_xfer_scd(gsc_io_t* io);
static	U32	_dmdma_rx_available_scd(gsc_io_t* gio);
static	U32	_dmdma_rx_setup_scd(gsc_io_t* gio);
static	U32	_dmdma_tx_available_scd(gsc_io_t* gio);
static	U32	_dmdma_tx_setup_scd(gsc_io_t* gio);
static	U32	_dmdma_tx_xfer_scd(gsc_io_t* io);
static	U32 _rx_done_underrun(gsc_io_t* gio);
static	U32	_rx_setup(gsc_io_t* gio);
static	U32	_tx_done_overrun(gsc_io_t* gio);
static	U32	_tx_setup(gsc_io_t* gio);



/* variables	*************************************************************/

// The API breaks requests into smaller chuncks so that all are done safely.
const gsc_io_func_t	io_dma_read_auto_funcs	=
{
	{
	/* _8.setup			*/	_rx_setup,
	/* _8.available		*/	io_pio_rx_available,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	gsc_io_dma_read,
	/* _8.transfer_reg	*/	gsc_io_dma_read,
	/* _8.wait			*/	_dma_read_wait,
	/* _8.done			*/	_rx_done_underrun
	},
	{
	/* _16.setup		*/	_rx_setup,
	/* _16.available	*/	io_pio_rx_available,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	gsc_io_dma_read,
	/* _16.transfer_reg	*/	gsc_io_dma_read,
	/* _16.wait			*/	_dma_read_wait,
	/* _16.done			*/	_rx_done_underrun
	},
	{
	/* _32.setup		*/	_rx_setup,
	/* _32.available	*/	io_pio_rx_available,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	gsc_io_dma_read,
	/* _32.transfer_reg	*/	gsc_io_dma_read,
	/* _32.wait			*/	_dma_read_wait,
	/* _32.done			*/	_rx_done_underrun
	}
};

// The application is responsible for making sure chunck sizes are OK.
const gsc_io_func_t	io_dma_read_manual_funcs	=
{
	{
	/* _8.setup			*/	_rx_setup,
	/* _8.available		*/	_dma_rx_available,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	gsc_io_dma_read,
	/* _8.transfer_reg	*/	gsc_io_dma_read,
	/* _8.wait			*/	_dma_read_wait,
	/* _8.done			*/	_rx_done_underrun
	},
	{
	/* _16.setup		*/	_rx_setup,
	/* _16.available	*/	_dma_rx_available,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	gsc_io_dma_read,
	/* _16.transfer_reg	*/	gsc_io_dma_read,
	/* _16.wait			*/	_dma_read_wait,
	/* _16.done			*/	_rx_done_underrun
	},
	{
	/* _32.setup		*/	_rx_setup,
	/* _32.available	*/	_dma_rx_available,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	gsc_io_dma_read,
	/* _32.transfer_reg	*/	gsc_io_dma_read,
	/* _32.wait			*/	_dma_read_wait,
	/* _32.done			*/	_rx_done_underrun
	}
};

// The API breaks requests into smaller chuncks so that all are done safely.
// The application is responsible for starting data transfer.
const gsc_io_func_t	io_dma_write_auto_asd_funcs	=
{
	{
	/* _8.setup			*/	_tx_setup,
	/* _8.available		*/	io_pio_tx_available,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	gsc_io_dma_write,
	/* _8.transfer_reg	*/	gsc_io_dma_write,
	/* _8.wait			*/	_dma_write_wait,
	/* _8.done			*/	_tx_done_overrun
	},
	{
	/* _16.setup		*/	_tx_setup,
	/* _16.available	*/	io_pio_tx_available,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	gsc_io_dma_write,
	/* _16.transfer_reg	*/	gsc_io_dma_write,
	/* _16.wait			*/	_dma_write_wait,
	/* _16.done			*/	_tx_done_overrun
	},
	{
	/* _32.setup		*/	_tx_setup,
	/* _32.available	*/	io_pio_tx_available,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	gsc_io_dma_write,
	/* _32.transfer_reg	*/	gsc_io_dma_write,
	/* _32.wait			*/	_dma_write_wait,
	/* _32.done			*/	_tx_done_overrun
	}
};

// The API breaks requests into smaller chuncks so that all are done safely.
// The API is also responsible for starting data transfer.
const gsc_io_func_t	io_dma_write_auto_ase_funcs	=
{
	{
	/* _8.setup			*/	_tx_setup,
	/* _8.available		*/	io_pio_tx_available,
	/* _8.auto_start	*/	io_auto_start,
	/* _8.transfer_map	*/	gsc_io_dma_write,
	/* _8.transfer_reg	*/	gsc_io_dma_write,
	/* _8.wait			*/	_dma_write_wait,
	/* _8.done			*/	_tx_done_overrun
	},
	{
	/* _16.setup		*/	_tx_setup,
	/* _16.available	*/	io_pio_tx_available,
	/* _16.auto_start	*/	io_auto_start,
	/* _16.transfer_map	*/	gsc_io_dma_write,
	/* _16.transfer_reg	*/	gsc_io_dma_write,
	/* _16.wait			*/	_dma_write_wait,
	/* _16.done			*/	_tx_done_overrun
	},
	{
	/* _32.setup		*/	_tx_setup,
	/* _32.available	*/	io_pio_tx_available,
	/* _32.auto_start	*/	io_auto_start,
	/* _32.transfer_map	*/	gsc_io_dma_write,
	/* _32.transfer_reg	*/	gsc_io_dma_write,
	/* _32.wait			*/	_dma_write_wait,
	/* _32.done			*/	_tx_done_overrun
	}
};

// The application is responsible for making sure chunck sizes are OK.
// The application is also responsible for starting data transfer.
const gsc_io_func_t	io_dma_write_manual_asd_funcs	=
{
	{
	/* _8.setup			*/	_tx_setup,
	/* _8.available		*/	_dma_tx_available,
	/* _8.auto_start	*/	io_auto_start,
	/* _8.transfer_map	*/	gsc_io_dma_write,
	/* _8.transfer_reg	*/	gsc_io_dma_write,
	/* _8.wait			*/	_dma_write_wait,
	/* _8.done			*/	_tx_done_overrun
	},
	{
	/* _16.setup		*/	_tx_setup,
	/* _16.available	*/	_dma_tx_available,
	/* _16.auto_start	*/	io_auto_start,
	/* _16.transfer_map	*/	gsc_io_dma_write,
	/* _16.transfer_reg	*/	gsc_io_dma_write,
	/* _16.wait			*/	_dma_write_wait,
	/* _16.done			*/	_tx_done_overrun
	},
	{
	/* _32.setup		*/	_tx_setup,
	/* _32.available	*/	_dma_tx_available,
	/* _32.auto_start	*/	io_auto_start,
	/* _32.transfer_map	*/	gsc_io_dma_write,
	/* _32.transfer_reg	*/	gsc_io_dma_write,
	/* _32.wait			*/	_dma_write_wait,
	/* _32.done			*/	_tx_done_overrun
	}
};

// The application is responsible for making sure chunck sizes are OK.
// The API is responsible for starting data transfer.
const gsc_io_func_t	io_dma_write_manual_ase_funcs	=
{
	{
	/* _8.setup			*/	_tx_setup,
	/* _8.available		*/	_dma_tx_available,
	/* _8.auto_start	*/	io_auto_start,
	/* _8.transfer_map	*/	gsc_io_dma_write,
	/* _8.transfer_reg	*/	gsc_io_dma_write,
	/* _8.wait			*/	_dma_write_wait,
	/* _8.done			*/	_tx_done_overrun
	},
	{
	/* _16.setup		*/	_tx_setup,
	/* _16.available	*/	_dma_tx_available,
	/* _16.auto_start	*/	io_auto_start,
	/* _16.transfer_map	*/	gsc_io_dma_write,
	/* _16.transfer_reg	*/	gsc_io_dma_write,
	/* _16.wait			*/	_dma_write_wait,
	/* _16.done			*/	_tx_done_overrun
	},
	{
	/* _32.setup		*/	_tx_setup,
	/* _32.available	*/	_dma_tx_available,
	/* _32.auto_start	*/	io_auto_start,
	/* _32.transfer_map	*/	gsc_io_dma_write,
	/* _32.transfer_reg	*/	gsc_io_dma_write,
	/* _32.wait			*/	_dma_write_wait,
	/* _32.done			*/	_tx_done_overrun
	}
};

// Single Cycle Absent
// When the Rx FIFO falls to Almost Empty, transfer from the FIFO is paused.
const gsc_io_func_t	io_dmdma_read_sca_funcs	=
{
	{
	/* _8.setup			*/	_rx_setup,
	/* _8.available		*/	_dmdma_rx_available_scd,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	_dmdma_rx_xfer_scd,
	/* _8.transfer_reg	*/	_dmdma_rx_xfer_scd,
	/* _8.wait			*/	_dma_read_wait,
	/* _8.done			*/	_rx_done_underrun
	},
	{
	/* _16.setup		*/	_rx_setup,
	/* _16.available	*/	_dmdma_rx_available_scd,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	_dmdma_rx_xfer_scd,
	/* _16.transfer_reg	*/	_dmdma_rx_xfer_scd,
	/* _16.wait			*/	_dma_read_wait,
	/* _16.done			*/	_rx_done_underrun
	},
	{
	/* _32.setup		*/	_rx_setup,
	/* _32.available	*/	_dmdma_rx_available_scd,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	_dmdma_rx_xfer_scd,
	/* _32.transfer_reg	*/	_dmdma_rx_xfer_scd,
	/* _32.wait			*/	_dma_read_wait,
	/* _32.done			*/	_rx_done_underrun
	}
};

// Single Cycle Disable
// When the Rx FIFO falls to Almost Empty, transfer from the FIFO is paused.
const gsc_io_func_t	io_dmdma_read_scd_funcs	=
{
	{
	/* _8.setup			*/	_dmdma_rx_setup_scd,
	/* _8.available		*/	_dmdma_rx_available_scd,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	_dmdma_rx_xfer_scd,
	/* _8.transfer_reg	*/	_dmdma_rx_xfer_scd,
	/* _8.wait			*/	_dma_read_wait,
	/* _8.done			*/	_rx_done_underrun
	},
	{
	/* _16.setup		*/	_dmdma_rx_setup_scd,
	/* _16.available	*/	_dmdma_rx_available_scd,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	_dmdma_rx_xfer_scd,
	/* _16.transfer_reg	*/	_dmdma_rx_xfer_scd,
	/* _16.wait			*/	_dma_read_wait,
	/* _16.done			*/	_rx_done_underrun
	},
	{
	/* _32.setup		*/	_dmdma_rx_setup_scd,
	/* _32.available	*/	_dmdma_rx_available_scd,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	_dmdma_rx_xfer_scd,
	/* _32.transfer_reg	*/	_dmdma_rx_xfer_scd,
	/* _32.wait			*/	_dma_read_wait,
	/* _32.done			*/	_rx_done_underrun
	}
};

// Single Cycle Present
// When the Rx FIFO falls to Almost Empty, transfer from the FIFO uses Single Cycles.
const gsc_io_func_t	io_dmdma_read_scp_funcs	=
{
	{
	/* _8.setup			*/	_rx_setup,
	/* _8.available		*/	io_pio_rx_available,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	gsc_io_dma_read,
	/* _8.transfer_reg	*/	gsc_io_dma_read,
	/* _8.wait			*/	_dma_read_wait,
	/* _8.done			*/	_rx_done_underrun
	},
	{
	/* _16.setup		*/	_rx_setup,
	/* _16.available	*/	io_pio_rx_available,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	gsc_io_dma_read,
	/* _16.transfer_reg	*/	gsc_io_dma_read,
	/* _16.wait			*/	_dma_read_wait,
	/* _16.done			*/	_rx_done_underrun
	},
	{
	/* _32.setup		*/	_rx_setup,
	/* _32.available	*/	io_pio_rx_available,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	gsc_io_dma_read,
	/* _32.transfer_reg	*/	gsc_io_dma_read,
	/* _32.wait			*/	_dma_read_wait,
	/* _32.done			*/	_rx_done_underrun
	}
};

// Single Cycle Absent, Auto-Start Disable
// When the Tx FIFO fills to Almost Full, transfer to the FIFO is paused.
const gsc_io_func_t	io_dmdma_write_sca_asd_funcs	=
{
	{
	/* _8.setup			*/	_tx_setup,
	/* _8.available		*/	_dmdma_tx_available_scd,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	_dmdma_tx_xfer_scd,
	/* _8.transfer_reg	*/	_dmdma_tx_xfer_scd,
	/* _8.wait			*/	_dma_write_wait,
	/* _8.done			*/	_tx_done_overrun
	},
	{
	/* _16.setup		*/	_tx_setup,
	/* _16.available	*/	_dmdma_tx_available_scd,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	_dmdma_tx_xfer_scd,
	/* _16.transfer_reg	*/	_dmdma_tx_xfer_scd,
	/* _16.wait			*/	_dma_write_wait,
	/* _16.done			*/	_tx_done_overrun
	},
	{
	/* _32.setup		*/	_tx_setup,
	/* _32.available	*/	_dmdma_tx_available_scd,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	_dmdma_tx_xfer_scd,
	/* _32.transfer_reg	*/	_dmdma_tx_xfer_scd,
	/* _32.wait			*/	_dma_write_wait,
	/* _32.done			*/	_tx_done_overrun
	}
};

// Single Cycle Absent, Auto-Start Enable
// When the Tx FIFO fills to Almost Full, transfer to the FIFO is paused.
const gsc_io_func_t	io_dmdma_write_sca_ase_funcs	=
{
	{
	/* _8.setup			*/	_tx_setup,
	/* _8.available		*/	_dmdma_tx_available_scd,
	/* _8.auto_start	*/	io_auto_start,
	/* _8.transfer_map	*/	_dmdma_tx_xfer_scd,
	/* _8.transfer_reg	*/	_dmdma_tx_xfer_scd,
	/* _8.wait			*/	_dma_write_wait,
	/* _8.done			*/	_tx_done_overrun
	},
	{
	/* _16.setup		*/	_tx_setup,
	/* _16.available	*/	_dmdma_tx_available_scd,
	/* _16.auto_start	*/	io_auto_start,
	/* _16.transfer_map	*/	_dmdma_tx_xfer_scd,
	/* _16.transfer_reg	*/	_dmdma_tx_xfer_scd,
	/* _16.wait			*/	_dma_write_wait,
	/* _16.done			*/	_tx_done_overrun
	},
	{
	/* _32.setup		*/	_tx_setup,
	/* _32.available	*/	_dmdma_tx_available_scd,
	/* _32.auto_start	*/	io_auto_start,
	/* _32.transfer_map	*/	_dmdma_tx_xfer_scd,
	/* _32.transfer_reg	*/	_dmdma_tx_xfer_scd,
	/* _32.wait			*/	_dma_write_wait,
	/* _32.done			*/	_tx_done_overrun
	}
};

// Single Cycle Disable, Auto-Start Disable
// When the Tx FIFO fills to Almost Full, transfer to the FIFO is paused.
const gsc_io_func_t	io_dmdma_write_scd_asd_funcs	=
{
	{
	/* _8.setup			*/	_dmdma_tx_setup_scd,
	/* _8.available		*/	_dmdma_tx_available_scd,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	_dmdma_tx_xfer_scd,
	/* _8.transfer_reg	*/	_dmdma_tx_xfer_scd,
	/* _8.wait			*/	_dma_write_wait,
	/* _8.done			*/	_tx_done_overrun
	},
	{
	/* _16.setup		*/	_dmdma_tx_setup_scd,
	/* _16.available	*/	_dmdma_tx_available_scd,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	_dmdma_tx_xfer_scd,
	/* _16.transfer_reg	*/	_dmdma_tx_xfer_scd,
	/* _16.wait			*/	_dma_write_wait,
	/* _16.done			*/	_tx_done_overrun
	},
	{
	/* _32.setup		*/	_dmdma_tx_setup_scd,
	/* _32.available	*/	_dmdma_tx_available_scd,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	_dmdma_tx_xfer_scd,
	/* _32.transfer_reg	*/	_dmdma_tx_xfer_scd,
	/* _32.wait			*/	_dma_write_wait,
	/* _32.done			*/	_tx_done_overrun
	}
};

// Single Cycle Disable, Auto-Start Enable
// When the Tx FIFO fills to Almost Full, transfer to the FIFO is paused.
const gsc_io_func_t	io_dmdma_write_scd_ase_funcs	=
{
	{
	/* _8.setup			*/	_dmdma_tx_setup_scd,
	/* _8.available		*/	_dmdma_tx_available_scd,
	/* _8.auto_start	*/	io_auto_start,
	/* _8.transfer_map	*/	_dmdma_tx_xfer_scd,
	/* _8.transfer_reg	*/	_dmdma_tx_xfer_scd,
	/* _8.wait			*/	_dma_write_wait,
	/* _8.done			*/	_tx_done_overrun
	},
	{
	/* _16.setup		*/	_dmdma_tx_setup_scd,
	/* _16.available	*/	_dmdma_tx_available_scd,
	/* _16.auto_start	*/	io_auto_start,
	/* _16.transfer_map	*/	_dmdma_tx_xfer_scd,
	/* _16.transfer_reg	*/	_dmdma_tx_xfer_scd,
	/* _16.wait			*/	_dma_write_wait,
	/* _16.done			*/	_tx_done_overrun
	},
	{
	/* _32.setup		*/	_dmdma_tx_setup_scd,
	/* _32.available	*/	_dmdma_tx_available_scd,
	/* _32.auto_start	*/	io_auto_start,
	/* _32.transfer_map	*/	_dmdma_tx_xfer_scd,
	/* _32.transfer_reg	*/	_dmdma_tx_xfer_scd,
	/* _32.wait			*/	_dma_write_wait,
	/* _32.done			*/	_tx_done_overrun
	}
};

// Single Cycle Present, Auto-Start Disable
// When the Tx FIFO fills to Almost Full, transfer to the FIFO uses Single Cycles.
const gsc_io_func_t	io_dmdma_write_scp_asd_funcs	=
{
	{
	/* _8.setup			*/	_tx_setup,
	/* _8.available		*/	io_pio_tx_available,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	gsc_io_dma_write,
	/* _8.transfer_reg	*/	gsc_io_dma_write,
	/* _8.wait			*/	_dma_write_wait,
	/* _8.done			*/	_tx_done_overrun
	},
	{
	/* _16.setup		*/	_tx_setup,
	/* _16.available	*/	io_pio_tx_available,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	gsc_io_dma_write,
	/* _16.transfer_reg	*/	gsc_io_dma_write,
	/* _16.wait			*/	_dma_write_wait,
	/* _16.done			*/	_tx_done_overrun
	},
	{
	/* _32.setup		*/	_tx_setup,
	/* _32.available	*/	io_pio_tx_available,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	gsc_io_dma_write,
	/* _32.transfer_reg	*/	gsc_io_dma_write,
	/* _32.wait			*/	_dma_write_wait,
	/* _32.done			*/	_tx_done_overrun
	}
};

// Single Cycle Present, Auto-Start Enable
// When the Tx FIFO fills to Almost Full, transfer to the FIFO uses Single Cycles.
const gsc_io_func_t	io_dmdma_write_scp_ase_funcs	=
{
	{
	/* _8.setup			*/	_tx_setup,
	/* _8.available		*/	io_pio_tx_available,
	/* _8.auto_start	*/	io_auto_start,
	/* _8.transfer_map	*/	gsc_io_dma_write,
	/* _8.transfer_reg	*/	gsc_io_dma_write,
	/* _8.wait			*/	_dma_write_wait,
	/* _8.done			*/	_tx_done_overrun
	},
	{
	/* _16.setup		*/	_tx_setup,
	/* _16.available	*/	io_pio_tx_available,
	/* _16.auto_start	*/	io_auto_start,
	/* _16.transfer_map	*/	gsc_io_dma_write,
	/* _16.transfer_reg	*/	gsc_io_dma_write,
	/* _16.wait			*/	_dma_write_wait,
	/* _16.done			*/	_tx_done_overrun
	},
	{
	/* _32.setup		*/	_tx_setup,
	/* _32.available	*/	io_pio_tx_available,
	/* _32.auto_start	*/	io_auto_start,
	/* _32.transfer_map	*/	gsc_io_dma_write,
	/* _32.transfer_reg	*/	gsc_io_dma_write,
	/* _32.wait			*/	_dma_write_wait,
	/* _32.done			*/	_tx_done_overrun
	}
};



/******************************************************************************
*
*	Function:	_dma_info_init
*
*	Purpose:
*
*		Initialize the referenced information structure in preparation for a
*		Non-Demand Mode DMA transfer.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*		desc		The structure to initialize.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dma_info_init(gsc_io_t* gio, gsc_dma_info_t* info)
{
	/*
	*	These settings are specific to the HPDI32 and may be different for
	*	other boards.
	*/

	memset(info, 0, sizeof(info[0]));
	info->ready_input		= 1;
	info->bterm_input		= 0;
	info->local_bursting	= 1;
	info->wr_inval_mode		= 1;
	info->dma_eot_pin		= 0;
	info->demand_mode		= gio->dma_demand ? 1 : 0;
	info->stop_xfer_mode	= 0;
	info->fifo_mode			= 1;
	info->wait_states		= 0;
	info->bus_width			= (int) gio->data_size;

	if (gio->dev->tx.gio.dma_priority == gio->dev->rx.gio.dma_priority)
	{
		// Equal priority.
		info->priority	= GSC_DMA_PRI_SHARE;
	}
	else if (gio->dma_priority == HPDI32_IO_DMA_PRIORITY_ENABLE)
	{
		// This channel has priority.

		if (gio->dma_channel->index == 0)
			info->priority	= GSC_DMA_PRI_CH_0;
		else
			info->priority	= GSC_DMA_PRI_CH_1;
	}
	else
	{
		// The other channel has priority.

		if (gio->dma_channel->index == 0)
			info->priority	= GSC_DMA_PRI_CH_1;
		else
			info->priority	= GSC_DMA_PRI_CH_0;
	}

	return((U32) GSC_SUCCESS);
}



/******************************************************************************
*
*	Function:	_dma_read_wait
*
*	Purpose:
*
*		Wait for data to become available to read.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*		timeout_ms	The amount of time left to wait.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dma_read_wait(gsc_io_t* gio, U32 timeout_ms)
{
	#define	_IRQ_	HPDI32_IRQ_RX_AE
	U32	status;

	if (gio->xfer)
	{
		// Wait for the DMA transfer to complete.
		// Wait for the DMA Done interrupt.
		status	= gsc_io_dma_wait(gio, timeout_ms);
	}
	else
	{
		// Wait for data to become available for reading.
		// Wait for the Rx FIFO Not Almost Empty interrupt.
		status	= irq_interrupt_disable(gio->dev, _IRQ_);

		if (status == (U32) GSC_SUCCESS)
			status	= irq_interrupt_trigger_config(gio->dev, _IRQ_, ~0, ~0);

		if (status == (U32) GSC_SUCCESS)
			status	= irq_interrupt_enable(gio->dev, _IRQ_);

		if (status != (U32) GSC_SUCCESS)
		{
		}
		else if (timeout_ms > 100)
		{
			status	= gsc_event_wait(gio->dev->irq.locals[13], 100);

			if (status == (U32) GSC_WAIT_TIMEOUT)
				status	= (U32) GSC_SUCCESS;
		}
		else
		{
			status	= gsc_event_wait(gio->dev->irq.locals[13], timeout_ms);
		}
	}

	return(status);
	#undef	_IRQ_
}



/******************************************************************************
*
*	Function:	_dma_rx_available
*
*	Purpose:
*
*		Determine the number of available bytes, assuming 8-bit, 16-bit or
*		32-bit samples.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dma_rx_available(gsc_io_t* gio)
{
	U32	status;

	if (gio->timeout_s)
	{
		status		= (U32) GSC_SUCCESS;
		gio->xfer	= gio->xfer_tot - gio->xfered_tot;
	}
	else
	{
		status	= io_pio_rx_available(gio);
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_dmdma_rx_available_scd
*
*	Purpose:
*
*		Determine the number of available bytes, assuming 8-bit, 16-bit or
*		32-bit samples. This is done under the condition that we have to
*		account for the Single Cycle Disable feature. This is also applicable
*		when the board doesn't include Single Cycle support.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dmdma_rx_available_scd(gsc_io_t* gio)
{
	U32	ae;
	U32	rar;
	U32	samples;
	U32	size;
	U32	status;

	for (;;)	// A convenience loop.
	{
		if (gio->timeout_s == 0)
		{
			// Get what is there right now and quit.
			gio->dev->rx.use_pio	= 1;
			status	= io_pio_rx_available(gio);
			break;
		}

		status	= GSC_REG_READ(gio->dev, HPDI32_RAR, &rar);
		if (status != (U32) GSC_SUCCESS)	break;
		size	= gio->data_size / 8;
		samples	= (gio->xfer_tot - gio->xfered_tot) / size;
		ae		= HPDI32_RAR_DECODE_EMPTY(rar);

		if (samples > ae)
		{
			// Read upto the Almost Empty point, trying to avoid a timeout.
			gio->xfer	= (gio->xfer_tot - gio->xfered_tot) - (ae * size);
			break;
		}

		/*
		*	Since the Almost Empty level is usually relatively low, it should
		*	be more efficient to get the data via PIO, rather than perform a
		*	small DMDMA. This should be so even if there is enough data in
		*	the Rx FIFO to complete a small DMDMA.
		*/

		gio->dev->rx.use_pio	= 1;
		status	= io_pio_rx_available(gio);

		// Make sure we're using the user space address. If an API buffer is
		// being used, then the transfer buffer is presently a physical
		// address.
		gio->xfer_buf = (void*) (gio->xfered_tot + (UL) gio->buffer);
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_dma_tx_available
*
*	Purpose:
*
*		Determine the number of available spaces, assuming 8-bit, 16-bit or
*		32-bit samples.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dma_tx_available(gsc_io_t* gio)
{
	U32	status;

	if (gio->timeout_s)
	{
		status		= (U32) GSC_SUCCESS;
		gio->xfer	= gio->xfer_tot - gio->xfered_tot;
	}
	else
	{
		status	= io_pio_tx_available(gio);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_dma_setup
*
*	Purpose:
*
*		Perform any needed setup in preparation for a DMA I/O operation which
*		is about to begin.
*
*	Arguments:
*
*		gio			The common I/O data structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dma_setup(gsc_io_t* gio)
{
	gsc_mem_t	mem;
	U32			set;
	U32			status;

	status	= gsc_mem_test(&gio->dev->tx.mem, gio->buffer, gio->xfer_tot, &mem);

	if ((status == (U32) GSC_SUCCESS) && (mem.size == 0))
		status	= gsc_mem_test(&gio->dev->rx.mem, gio->buffer, gio->xfer_tot, &mem);

	if (status != (U32) GSC_SUCCESS)
	{
	}
	else if (mem.size)
	{
		// Block mode DMA/DMDMA
		// Don't modify gio->buffer as it may be needed as is later on.
		gio->xfer_buf	= (void*) (UL) mem.phy_adrs;
		status			= gsc_dma_channel_open(	&gio->dev->dma,
												&gio->dma_channel,
												GSC_DMA_TYPE_BLOCK,
												gio->dma_need,
												_dma_info_init,
												gio);
	}
	else
	{
		// Scatter-Gather mode DMA/DMDMA
		status	= gsc_dma_channel_open(	&gio->dev->dma,
										&gio->dma_channel,
										GSC_DMA_TYPE_CHAINED,
										gio->dma_need,
										_dma_info_init,
										gio);
	}

	if ((status == (U32) GSC_SUCCESS) &&
		(gio->dma_demand) &&
		(gio->dma_channel->index == 0))
	{
		set		= gio->is_tx ? HPDI32_BCR_DMDMA0_DIR : 0;
		status	= GSC_REG_MOD(	gio->dev,
								HPDI32_BCR,
								set,
								HPDI32_BCR_DMDMA0_DIR);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_rx_setup
*
*	Purpose:
*
*		Perform any needed setup in preparation for an Rx DMA I/O operation
*		which is about to begin.
*
*	Arguments:
*
*		gio			The common I/O data structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _rx_setup(gsc_io_t* gio)
{
	U32	status;

	gio->dev->rx.use_pio	= 0;
	status	= _dma_setup(gio);
	return(status);
}



/*****************************************************************************
*
*	Function:	_tx_setup
*
*	Purpose:
*
*		Perform any needed setup in preparation for a Tx DMA I/O operation
*		which is about to begin.
*
*	Arguments:
*
*		gio			The common I/O data structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _tx_setup(gsc_io_t* gio)
{
	U32	status;

	gio->dev->tx.use_pio	= 0;
	status	= _dma_setup(gio);
	return(status);
}



/*****************************************************************************
*
*	Function:	_dmdma_rx_setup_scd
*
*	Purpose:
*
*		Perform any needed setup in preparation for a DMDMA I/O read operation
*		which is about to begin. This version performs additional work to
*		support the Single Cycle Disable feature.
*
*	Arguments:
*
*		gio			The common I/O data structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dmdma_rx_setup_scd(gsc_io_t* gio)
{
	U32	status;

	status	= GSC_REG_MOD(gio->dev, HPDI32_BCR, 0xFFFFFFFF, HPDI32_BCR_SING_CYC_DIS);

	if (status == (U32) GSC_SUCCESS)
		status	= _rx_setup(gio);

	return(status);
}



/******************************************************************************
*
*	Function:	_dmdma_rx_xfer_scd
*
*	Purpose:
*
*		Read data using Demand Mode DMA in accommodation of the Single Cycle
*		Disable feature. This means we may have to resort to PIO. This is also
*		applicable when firmware doesn't include Single Cycle support.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dmdma_rx_xfer_scd(gsc_io_t* gio)
{
	U32	status;

	if (gio->dev->rx.use_pio == 0)
	{
		status	= gsc_io_dma_read(gio);
	}
	else if (gio->data_size == 32)
	{
		if (gio->bar->ptr)
			status	= io_pio_read_funcs._32.transfer_map(gio);
		else
			status	= io_pio_read_funcs._32.transfer_reg(gio);
	}
	else if (gio->data_size == 16)
	{
		if (gio->bar->ptr)
			status	= io_pio_read_funcs._16.transfer_map(gio);
		else
			status	= io_pio_read_funcs._16.transfer_reg(gio);
	}
	else
	{
		if (gio->bar->ptr)
			status	= io_pio_read_funcs._8.transfer_map(gio);
		else
			status	= io_pio_read_funcs._8.transfer_reg(gio);
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_dmdma_tx_available_scd
*
*	Purpose:
*
*		Determine the number of available bytes that can be transferred,
*		assuming 8-bit, 16-bit or 32-bit samples. This is done under the
*		condition that we have to account for the Single Cycle Disable
*		feature.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dmdma_tx_available_scd(gsc_io_t* gio)
{
	U32	af;
	U32	samples;
	U32	size;
	U32	status;
	U32	tar;

	for (;;)	// A convenience loop.
	{
		if (gio->timeout_s == 0)
		{
			// Get what is there right now and quit.
			gio->dev->tx.use_pio	= 1;
			status	= io_pio_tx_available(gio);
			break;
		}

		status	= GSC_REG_READ(gio->dev, HPDI32_TAR, &tar);
		if (status != (U32) GSC_SUCCESS)	break;
		size	= gio->data_size / 8;
		samples	= (gio->xfer_tot - gio->xfered_tot) / size;
		af		= HPDI32_TAR_DECODE_EMPTY(tar);

		if (samples > af)
		{
			// Write upto the Almost Full point, trying to avoid a timeout.
			gio->xfer	= (gio->xfer_tot - gio->xfered_tot) - (af * size);
			break;
		}

		/*
		*	Since the Almost Full level is usually relatively low, it should
		*	be more efficient to send the data via PIO, rather than perform a
		*	small DMDMA. This should be so even if there is enough space in
		*	the Tx FIFO to complete a small DMDMA.
		*/

		gio->dev->tx.use_pio	= 1;
		status	= io_pio_tx_available(gio);

		// Make sure we're using the user space address. If an API buffer is
		// being used, then the transfer buffer is presently a physical
		// address.
		gio->xfer_buf = (void*) (gio->xfered_tot + (UL) gio->buffer);
		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_dmdma_tx_setup_scd
*
*	Purpose:
*
*		Perform any needed setup in preparation for a DMDMA I/O write
*		operation which is about to begin. This version performs additional
*		work to support the Single Cycle Disable feature.
*
*	Arguments:
*
*		gio			The common I/O data structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dmdma_tx_setup_scd(gsc_io_t* gio)
{
	U32	status;

	status	= GSC_REG_MOD(gio->dev, HPDI32_BCR, 0xFFFFFFFF, HPDI32_BCR_SING_CYC_DIS);

	if (status == (U32) GSC_SUCCESS)
		status	= _tx_setup(gio);

	return(status);
}



/******************************************************************************
*
*	Function:	_dmdma_tx_xfer_scd
*
*	Purpose:
*
*		Write data using Demand Mode DMA in accommodation of the Single Cycle
*		Disable feature. This means we may have to resort to PIO. This is also
*		applicable for when firmware doesn't do Single Cycle transfers.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dmdma_tx_xfer_scd(gsc_io_t* gio)
{
	U32	status;

	if (gio->dev->tx.use_pio == 0)
	{
		status	= gsc_io_dma_write(gio);
	}
	else if (gio->data_size == 32)
	{
		if (gio->bar->ptr)
		{
			if (gio->auto_start)
				status	= io_pio_write_ase_funcs._32.transfer_map(gio);
			else
				status	= io_pio_write_asd_funcs._32.transfer_map(gio);
		}
		else
		{
			if (gio->auto_start)
				status	= io_pio_write_ase_funcs._32.transfer_reg(gio);
			else
				status	= io_pio_write_asd_funcs._32.transfer_reg(gio);
		}
	}
	else if (gio->data_size == 16)
	{
		if (gio->bar->ptr)
		{
			if (gio->auto_start)
				status	= io_pio_write_ase_funcs._16.transfer_map(gio);
			else
				status	= io_pio_write_asd_funcs._16.transfer_map(gio);
		}
		else
		{
			if (gio->auto_start)
				status	= io_pio_write_ase_funcs._16.transfer_reg(gio);
			else
				status	= io_pio_write_asd_funcs._16.transfer_reg(gio);
		}
	}
	else
	{
		if (gio->bar->ptr)
		{
			if (gio->auto_start)
				status	= io_pio_write_ase_funcs._8.transfer_map(gio);
			else
				status	= io_pio_write_asd_funcs._8.transfer_map(gio);
		}
		else
		{
			if (gio->auto_start)
				status	= io_pio_write_ase_funcs._8.transfer_reg(gio);
			else
				status	= io_pio_write_asd_funcs._8.transfer_reg(gio);
		}
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_dma_write_wait
*
*	Purpose:
*
*		Wait for write space to become available to write.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*		timeout_ms	The amount of time left to wait.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dma_write_wait(gsc_io_t* gio, U32 timeout_ms)
{
	#define	_IRQ_	HPDI32_IRQ_TX_AF
	U32	delta;
	U32	status;

	if (gio->xfer)
	{
		if (gio->timeout_s == 0)
		{
			// Wait for the data that was written to transfer.
			timeout_ms	= 7;	// 128K @ 20MHz
		}

		if ((gio->auto_start) &&
			((gio->dev->cache.fsr & HPDI32_FSR_GPIO6_TXAS) == 0))
		{
			// While waiting for the DMA write to complete we have to set Tx
			// Start periodically as it may clear if the Tx FIFO momentarily
			// goes empty. If that were to happen, and it has been seen, then
			// the DMA could timeout if the remainder of the DMA exceeds the
			// FIFO size.

			for (;;)
			{
				GSC_REG_MOD(gio->dev, HPDI32_BCR, ~0, HPDI32_BCR_TX_START);

				if (timeout_ms <= 5)
				{
					status	= gsc_io_dma_wait(gio, timeout_ms);
					GSC_REG_MOD(gio->dev, HPDI32_BCR, ~0, HPDI32_BCR_TX_START);
					break;
				}

				delta	= os_time_delta_ms();
				status	= gsc_io_dma_wait(gio, 1);
				GSC_REG_MOD(gio->dev, HPDI32_BCR, ~0, HPDI32_BCR_TX_START);

				if (status == (U32) GSC_SUCCESS)
					break;

				delta	= os_time_delta_ms() - delta;

				if (delta > timeout_ms)
					break;

				timeout_ms	-= delta;
			}
		}
		else
		{
			// Wait for the DMA transfer to complete.
			// Wait for the DMA Done interrupt.
			status	= gsc_io_dma_wait(gio, timeout_ms);
		}
	}
	else
	{
		// Wait for space to become available for writing.
		// Wait for the Tx FIFO Not Almost Full interrupt.
		status	= irq_interrupt_disable(gio->dev, _IRQ_);

		if (status == (U32) GSC_SUCCESS)
			status	= irq_interrupt_trigger_config(gio->dev, _IRQ_, ~0, ~0);

		if (status == (U32) GSC_SUCCESS)
			status	= irq_interrupt_enable(gio->dev, _IRQ_);

		if (status != (U32) GSC_SUCCESS)
		{
		}
		else if (timeout_ms > 100)
		{
			status	= gsc_event_wait(gio->dev->irq.locals[10], 100);

			if (status == (U32) GSC_WAIT_TIMEOUT)
				status	= (U32) GSC_SUCCESS;
		}
		else
		{
			status	= gsc_event_wait(gio->dev->irq.locals[10], timeout_ms);
		}
	}

	return(status);
	#undef	_IRQ_
}



/******************************************************************************
*
*	Function:	_rx_done_underrun
*
*	Purpose:
*
*		Perform any post processing of a DMDMA read request.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _rx_done_underrun(gsc_io_t* gio)
{
	U32	bsr;
	U32	status;

	status	= gsc_dma_done(gio);

	if ((status == (U32) GSC_SUCCESS) &&
		(gio->dev->cache.fsr & HPDI32_FSR_OVR_UNDR_RUN))
	{
		status	= GSC_REG_READ(gio->dev, HPDI32_BSR, &bsr);

		if ((status == (U32) GSC_SUCCESS) && (bsr & HPDI32_BSR_RX_UNDER_RUN))
			status	= (U32) GSC_INVALID_DATA;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_tx_done_overrun
*
*	Purpose:
*
*		Perform any post processing of a DMA/DMDMA write request.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _tx_done_overrun(gsc_io_t* gio)
{
	U32	bsr;
	U32	status;

	status	= gsc_dma_done(gio);

	if ((status == (U32) GSC_SUCCESS) &&
		(gio->dev->cache.fsr & HPDI32_FSR_OVR_UNDR_RUN))
	{
		status	= GSC_REG_READ(gio->dev, HPDI32_BSR, &bsr);

		if ((status == (U32) GSC_SUCCESS) && (bsr & HPDI32_BSR_TX_OVERRUN))
			status	= (U32) GSC_INVALID_DATA;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	io_dma_init
*
*	Purpose:
*
*		Initialize the DMA data to its initial open() state.
*
*	Arguments:
*
*		dev			The device to initialize.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 io_dma_init(dev_data_t* dev)
{
	U32	ch0;
	U32	ch1;
	U32	stat;
	U32	status	= (U32) GSC_SUCCESS;

	stat	= gsc_dma_channel_close(&dev->dma, &dev->rx.gio.dma_channel);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_dma_channel_close(&dev->dma, &dev->tx.gio.dma_channel);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	if (dev->cache.fsr & HPDI32_FSR_DMDMA_CH1)
	{
		// Use CH0 for Rx, CH1 for Tx.
		// Channel zero really has full capabilities.
		ch0	= GSC_DMA_FLAG_BLDMA_RX
			| GSC_DMA_FLAG_DMDMA_RX;
		// Channel one does NOT do Rx DMDMA.
		ch1	= GSC_DMA_FLAG_BLDMA_TX
			| GSC_DMA_FLAG_DMDMA_TX;
	}
	else
	{
		ch0	= GSC_DMA_FLAG_BLDMA_TX
			| GSC_DMA_FLAG_BLDMA_RX
			| GSC_DMA_FLAG_DMDMA_TX
			| GSC_DMA_FLAG_DMDMA_RX;
		ch1	= GSC_DMA_FLAG_BLDMA_TX
			| GSC_DMA_FLAG_BLDMA_RX;
	}

	stat	= gsc_dma_init(&dev->dma, ch0, ch1);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	return(status);
}


