// API - HPDI32 - Any OS
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/dev_rx.c $
// $Rev: 23674 $
// $Date: 2013-10-31 16:25:34 -0500 (Thu, 31 Oct 2013) $

#include "dev_rx.h"

#include "dev_config.h"
#include "dev_reg.h"



/*****************************************************************************
*
*	Function:	rx_enable
*
*	Purpose:
*
*		Implement the Rx Enable parameter service. Validate and apply the
*		setting, as appropriate, then retrieve the current setting for the
*		caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 rx_enable(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_RX_ENABLE_DEFAULT;
		}
		else if (	(set == HPDI32_RX_ENABLE_YES) ||
					(set == HPDI32_RX_ENABLE_NO))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= HPDI32_RX_ENABLE_YES;
		}
		else
		{
			set	= HPDI32_RX_ENABLE_NO;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == HPDI32_RX_ENABLE_YES)
				set	= HPDI32_BCR_RX_ENABLED;
			else
				set	= 0;

			status	= GSC_REG_MOD(dev, HPDI32_BCR, set, HPDI32_BCR_RX_ENABLED);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= (bcr & HPDI32_BCR_RX_ENABLED)
					? HPDI32_RX_ENABLE_YES
					: HPDI32_RX_ENABLE_NO;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	rx_overrun
*
*	Purpose:
*
*		Implement the Rx Overrun parameter service. Validate and apply the
*		setting, as appropriate, then retrieve the current setting for the
*		caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 rx_overrun(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bsr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_RX_OVERRUN_DEFAULT;
		}
		else if (	(set == HPDI32_RX_OVERRUN_CLEAR) ||
					(set == HPDI32_RX_OVERRUN_IGNORE))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= HPDI32_RX_OVERRUN_CLEAR;
		}
		else
		{
			set	= HPDI32_RX_OVERRUN_IGNORE;
		}

		if ((set != GSC_NO_CHANGE) &&
			(set != HPDI32_RX_OVERRUN_IGNORE) &&
			(dev->cache.fsr & HPDI32_FSR_OVR_UNDR_RUN))
		{
			status	= GSC_REG_MOD(	dev,
									HPDI32_BSR,
									HPDI32_BSR_RX_OVERRUN,
									HPDI32_BSR_RX_OVERRUN |
									HPDI32_BSR_RX_UNDER_RUN |
									HPDI32_BSR_TX_OVERRUN);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			if (dev->cache.fsr & HPDI32_FSR_OVR_UNDR_RUN)
			{
				status	= GSC_REG_READ(dev, HPDI32_BSR, &bsr);
				if (status != (U32) GSC_SUCCESS)	break;

				get[0]	= (bsr & HPDI32_BSR_RX_OVERRUN)
						? HPDI32_RX_OVERRUN_YES
						: HPDI32_RX_OVERRUN_NO;
			}
			else
			{
				get[0]	= HPDI32_RX_OVERRUN_NO;
			}
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	rx_row_count
*
*	Purpose:
*
*		Implement the Rx Row Count parameter service. Validate and apply the
*		setting, as appropriate, then retrieve the current setting for the
*		caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 rx_row_count(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	rlcr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET is not supported for this parameter.

		if (get)
		{
			status	= GSC_REG_READ(dev, HPDI32_RLCR, &rlcr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= rlcr;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	rx_state
*
*	Purpose:
*
*		Implement the Rx State parameter service. Validate and apply the
*		setting, as appropriate, then retrieve the current setting for the
*		caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 rx_state(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET is not supported for this parameter.

		if (get)
		{
			if (bcr & HPDI32_BCR_RX_ENABLED)
				get[0]	= HPDI32_RX_STATE_ACTIVE;
			else
				get[0]	= HPDI32_RX_STATE_INACTIVE;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	rx_status_count
*
*	Purpose:
*
*		Implement the Rx Status Count parameter service. Validate and apply
*		the setting, as appropriate, then retrieve the current setting for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 rx_status_count(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	rscr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET is not supported for this parameter.

		if (get)
		{
			status	= GSC_REG_READ(dev, HPDI32_RSCR, &rscr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= rscr;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	rx_under_run
*
*	Purpose:
*
*		Implement the Rx Under Run parameter service. Validate and apply the
*		setting, as appropriate, then retrieve the current setting for the
*		caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 rx_under_run(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bsr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_RX_UNDER_RUN_DEFAULT;
		}
		else if (	(set == HPDI32_RX_UNDER_RUN_CLEAR) ||
					(set == HPDI32_RX_UNDER_RUN_IGNORE))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= HPDI32_RX_UNDER_RUN_CLEAR;
		}
		else
		{
			set	= HPDI32_RX_UNDER_RUN_IGNORE;
		}

		if ((set != GSC_NO_CHANGE) &&
			(set != HPDI32_RX_UNDER_RUN_IGNORE) &&
			(dev->cache.fsr & HPDI32_FSR_OVR_UNDR_RUN))
		{
			status	= GSC_REG_MOD(	dev,
									HPDI32_BSR,
									HPDI32_BSR_RX_UNDER_RUN,
									HPDI32_BSR_RX_OVERRUN |
									HPDI32_BSR_RX_UNDER_RUN |
									HPDI32_BSR_TX_OVERRUN);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			if (dev->cache.fsr & HPDI32_FSR_OVR_UNDR_RUN)
			{
				status	= GSC_REG_READ(dev, HPDI32_BSR, &bsr);
				if (status != (U32) GSC_SUCCESS)	break;

				get[0]	= (bsr & HPDI32_BSR_RX_UNDER_RUN)
						? HPDI32_RX_UNDER_RUN_YES
						: HPDI32_RX_UNDER_RUN_NO;
			}
			else
			{
				get[0]	= HPDI32_RX_UNDER_RUN_NO;
			}
		}

		break;
	}

	return(status);
}


