// API - HPDI32 - Any OS
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/dev_version.c $
// $Rev: 23674 $
// $Date: 2013-10-31 16:25:34 -0500 (Thu, 31 Oct 2013) $

#include "dev_version.h"

#include "gsc_version.h"

#include "dev_access.h"


/*****************************************************************************
*
*	Function:	dev_version_verify
*
*	Purpose:
*
*		Verify the version data for various components.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 dev_version_verify(void)
{
	U32	status;

	status	= gsc_version_verify();
	return(status);
}



/*****************************************************************************
*
*	Function:	hpdi32_version_get
*
*	Purpose:
*
*		Close an open communications channel.
*
*	Arguments:
*
*		handle		The handle of the device to be used to access the driver.
*
*		id			The item whose version number is desired.
*
*		version		The version number string is put here.
*
*		size		This is the size of the above buffer.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT hpdi32_version_get(void* handle, U8 id, char* version, size_t size)
{
	dev_data_t*	dev;
	U32			status;

	for (;;)	/* A convenience loop. */
	{
		if (version == NULL)
		{
			status	= (U32) GSC_NULL_PARAM;
			break;
		}

		if (size <= 1)
		{
			status	= (U32) GSC_INVALID_SIZE;
			break;
		}

		if (handle == NULL)
		{
			status	= gsc_version_get(NULL, id, version, size);
			break;
		}

		status	= dev_access_lock(handle, &dev);

		if (status == (U32) GSC_SUCCESS)
		{
			status	= gsc_version_get(&dev->device, id, version, size);
			dev_access_unlock(dev);
		}

		break;
	}

	return(status);
}


