// API - Any Device - Linux
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/gsc_os_sem.c $
// $Rev: 23674 $
// $Date: 2013-10-31 16:25:34 -0500 (Thu, 31 Oct 2013) $

#include <string.h>

#include "gsc_os_sem.h"



/*****************************************************************************
*
*	Function:	os_sem_create
*
*	Purpose:
*
*		Create a semaphore.
*
*	Arguments:
*
*		sem			The OS specific semaphore data is stored here.
*
*		name		A name to associate with the semaphore.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 os_sem_create(os_sem_t* sem, const char* name)
{
	U32	status;

	if ((sem) && (name) && (name[0]))
	{
		memset(sem, 0, sizeof(sem[0]));
		OS_KEY_SET(sem, OS_SEM_T_KEY);
		OS_SEM_T_KEY_VERIFY(sem);
		pthread_mutex_init(&sem->mutex, NULL);
		strncpy(sem->name, name, sizeof(sem->name));
		sem->name[sizeof(sem->name) -1]	= 0;
		OS_SEM_T_KEY_VERIFY(sem);
		status	= GSC_SUCCESS;
	}
	else
	{
		status	= GSC_NULL_PARAM;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	os_sem_destroy
*
*	Purpose:
*
*		Destroy a semaphore.
*
*	Arguments:
*
*		sem			The OS specific semaphore data is stored here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 os_sem_destroy(os_sem_t* sem)
{
	if (sem)
	{
		// Skip key validation as it may not have been initialized.
		os_sem_unlock(sem);
		pthread_mutex_destroy(&sem->mutex);
		memset(sem, 0, sizeof(os_sem_t));
	}

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	os_sem_lock
*
*	Purpose:
*
*		Acquire a semaphore.
*
*	Arguments:
*
*		sem			The OS specific semaphore data is stored here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 os_sem_lock(os_sem_t* sem)
{
	int	i;
	U32	status;

	if (sem)
	{
		OS_SEM_T_KEY_VERIFY(sem);
		i		= pthread_mutex_lock(&sem->mutex);
		status	= (i == 0) ? GSC_SUCCESS : GSC_FAILED;
		OS_SEM_T_KEY_VERIFY(sem);
	}
	else
	{
		status	= GSC_NULL_PARAM;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	os_sem_unlock
*
*	Purpose:
*
*		sem			The OS specific semaphore data is stored here.
*
*	Arguments:
*
*		sem		The semaphore to acquire.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void os_sem_unlock(os_sem_t* sem)
{
	if (sem)
	{
		// Skip key validation as it may not have been initialized.
		pthread_mutex_unlock(&sem->mutex);
	}
}



