// This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL).
// Any Device - Driver - Linux
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/driver/kernel_3.c $
// $Rev: 23671 $
// $Date: 2013-10-31 16:23:27 -0500 (Thu, 31 Oct 2013) $
//
//	Provide special handling for things specific to the 2.6 kernel.
//

#include <linux/module.h>
#include <linux/version.h>



#if	(LINUX_VERSION_CODE >= KERNEL_VERSION(3,0,0)) && \
	(LINUX_VERSION_CODE <  KERNEL_VERSION(4,0,0))

#include "main.h"



// Tags ***********************************************************************

MODULE_AUTHOR("General Standards Corporation, www.generalstandards.com");
MODULE_DESCRIPTION(	"GSC " DRV_DEV_DEVICE_NAME_UPPER
					" driver " DRV_GSC_FILE_VERSION_STR);
MODULE_LICENSE("GPL");
MODULE_SUPPORTED_DEVICE("GSC " DRV_DEV_DEVICE_NAME_UPPER);



/******************************************************************************
*
*	Function:	dma_sgl_user_mem_init
*
*	Purpose:
*
*		Initialize things in preparation for future kernel access of user
*		memory for SGL DMA operations.
*
*	Arguments:
*
*		dma		The DMA structure for which any initialization should be done.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dma_sgl_user_mem_init(dev_dma_t* dma)
{
	size_t	bytes	= sizeof(dma->sgl.iobuf[0]);

	if (dma)
	{
		dma->sgl.iobuf	= (void*) kmalloc(bytes, GFP_KERNEL);

		if (dma->sgl.iobuf)
			memset(dma->sgl.iobuf, 0, bytes);
	}
}



/******************************************************************************
*
*	Function:	dma_sgl_user_mem_map
*
*	Purpose:
*
*		Map user memory to kernel space for an SGL DMA transfer.
*
*	Arguments:
*
*		dma		The DMA structure with the references to the user memory
*				whose references are to be freed.
*
*		xfer	The data for the DMA transfer to be performed.
*
*	Returned:
*
*		0		All went well.
*		<0		An error code for the problem seen.
*
******************************************************************************/

int dma_sgl_user_mem_map(dev_dma_t* dma, gsc_drv_dma_t* xfer)
{
	unsigned long	bytes;
	u32				dpr;
	u32				dpr_flags;
	int				inc;
	long			l;
	u32				ladr;
	unsigned long	offset;
	unsigned long	padr;
	struct page*	page;
	unsigned long	pages;
	int				qty;
	u32				siz;
	int				status;
	void*			vi_adrs;
	int				wt;

	for (;;)
	{
		if (dma->sgl.list == NULL)
		{
			// We didn't get memory for the SGL DPR list.
			status			= -ENOMEM;
			xfer->status	= GSC_DMA_SGL_BUILD_FAILED;
			break;
		}

		if (dma->sgl.iobuf == NULL)
		{
			// We weren't able to get the structure.
			status			= -ENOMEM;
			xfer->status	= GSC_DMA_SGL_BUILD_FAILED;
			break;
		}

		dma->sgl.iobuf->flags		= xfer->u.xfer.flags;
		dma->sgl.iobuf->length		= xfer->u.xfer.bytes;
		dma->sgl.iobuf->offset		= (int) ((unsigned long)
										xfer->u.xfer.user_adrs
										& ~PAGE_MASK);
		dma->sgl.iobuf->nr_pages	= (	+ dma->sgl.iobuf->length
										+ dma->sgl.iobuf->offset
										- 1
										+ ~PAGE_MASK)
									>> PAGE_SHIFT;
		wt	= (xfer->u.xfer.flags & GSC_DMA_XFER_DIR_TO_HOST) ? 1 : 0;
		down_read(&current->mm->mmap_sem);
		qty	= get_user_pages(	current,
								current->mm,
								xfer->u.xfer.user_adrs,
								dma->sgl.iobuf->nr_pages,
								wt,
								0,
								&dma->sgl.iobuf->maplist[0],
								NULL);
		up_read(&current->mm->mmap_sem);
		dma->sgl.iobuf->mapped	= qty;

		if (qty <= 0)
		{
			// We were unable to map user memory.
			status			= -ENOMEM;
			xfer->status	= GSC_DMA_SGL_BUILD_FAILED;
			break;
		}

		dma->sgl.user_mapped	= 1;

		if (qty < dma->sgl.iobuf->nr_pages)
		{
			// We couldn't get the entire buffer.
			status			= -ENOMEM;
			xfer->status	= GSC_DMA_SGL_BUILD_FAILED;
			break;
		}

		// Compute the lower DPR bits.
		dpr_flags	= DMA_DPR_DESC_LOC_PCI;

		if (xfer->u.xfer.flags & GSC_DMA_XFER_DIR_TO_HOST)
			dpr_flags	|= DMA_DPR_BOARD_TO_HOST;
		else
			dpr_flags	|= DMA_DPR_HOST_TO_BOARD;

		// Make other computations.
		bytes	= dma->sgl.iobuf->length;
		inc		= (dma->flags & GSC_DMA_OPEN_LADRS_INC) ? 1 : 0;
		ladr	= xfer->u.xfer.local_adrs;
		offset	= dma->sgl.iobuf->offset;
		pages	= dma->sgl.iobuf->nr_pages - 1;
		status	= 0;

		for (l = 0; l <= pages ; l++)
		{
			page	= dma->sgl.iobuf->maplist[l];
			vi_adrs	= page_address(page);
			padr	= __pa(vi_adrs) + offset;
			dpr		= (dma->sgl.list[l].dpr_32 & 0xFFFFFFF0)
					| dpr_flags;

			if (padr > (0xFFFFFFFFUL - PAGE_SIZE + 1))
				status	= -EINVAL;	// Must be a 32-bit address for PLX.

			if (l == pages)
			{
				// This is the last page, which could also be the first.
				siz	= bytes;
				dpr	|= DMA_DPR_END_OF_CHAIN;
			}
			else if (l)
			{
				// These are the middle pages.
				siz	= PAGE_SIZE;
			}
			else
			{
				// This is the first page.
				siz		= PAGE_SIZE - offset;
			}

			offset						= 0;
			bytes						-= siz;
			dma->sgl.list[l].padr_32	= padr;
			dma->sgl.list[l].ladr_32	= ladr;
			dma->sgl.list[l].siz_32		= siz;
			dma->sgl.list[l].dpr_32		= dpr;

			if (inc)
				ladr	+= siz;
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	dma_sgl_user_mem_reset
*
*	Purpose:
*
*		Reset things to cleanup for any activity after kernel access of user
*		memory for SGL DMA operations.
*
*	Arguments:
*
*		dma		The DMA structure for which any reset steps should be taken.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dma_sgl_user_mem_reset(dev_dma_t* dma)
{
	if ((dma) && (dma->sgl.iobuf))
	{
		kfree(dma->sgl.iobuf);
		dma->sgl.iobuf	= NULL;
	}
}



/******************************************************************************
*
*	Function:	dma_sgl_user_mem_unmap
*
*	Purpose:
*
*		Free all references to any user memory made for an SGL DMA transfer.
*
*	Arguments:
*
*		dma		The DMA structure with the references to the user memory
*				whose references are to be freed.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dma_sgl_user_mem_unmap(dev_dma_t* dma)
{
	u32				flags;
	int				i;
	struct page*	page;

	if ((dma) && (dma->sgl.iobuf))
	{
		flags	= dma->sgl.iobuf->flags & GSC_DMA_XFER_DIR_TO_HOST;

		for (i = dma->sgl.iobuf->mapped; i > 0;)
		{
			i--;
			page	= dma->sgl.iobuf->maplist[i];

			if ((flags) && (!PageReserved(page)))
				SetPageDirty(page);

			//page_cache_release(page);	// Not in 2.6.18 - RH FC6
			put_page(page);				// Call this instead.
		}
	}
}



/******************************************************************************
*
*	Function:	irq_isr
*
*	Purpose:
*
*		Service an interrupt.
*
*	Arguments:
*
*		irq		The interrupt number.
*
*		dev_id	The private data we've associated with the IRQ.
*
*		regs	Unused. No longer present as of 2.6.19.
*
*	Returned:
*
*		The status as to whether we handled the interrupt or not.
*
******************************************************************************/

irqreturn_t irq_isr(int irq, void* dev_id   ISR_ARG3)
{
	int			handled;
	irqreturn_t	ret;

	handled	= irq_isr_common(irq, dev_id);
	ret		= handled ? IRQ_HANDLED : IRQ_NONE;
	return(ret);
}



/******************************************************************************
*
*	Function:	mmap_fault
*
*	Purpose:
*
*		Implement the mmap fault() procedure.
*
*	Arguments:
*
*		vma		The virtual memory area of interest.
*
*		vmf		The fault structure.
*
*	Returned:
*
*		0		All went well.
*		else	There was a problem.
*
******************************************************************************/

#if MMAP_FAULT_INCLUDE

int mmap_fault(struct vm_area_struct* vma, struct vm_fault* vmf)
{
	int	ret;

	vmf->page	= NULL;
	mmap_nopage_common(vma, (unsigned long) vmf->virtual_address, &vmf->page);
	ret			= vmf->page ? 0 : VM_FAULT_SIGBUS;
	return(ret);
}

#endif



/******************************************************************************
*
*	Function:	mmap_nopage
*
*	Purpose:
*
*		Implement the mmap nopage() procedure.
*
*	Arguments:
*
*		vma		The virtual memory area of interest.
*
*		address	The virtual address of the page being accessed.
*
*		type	We record the mapping status here.
*
*	Returned:
*
*		NULL	The request failed.
*		else	The request succeeded.
*
******************************************************************************/

#if MMAP_NOPAGE_INCLUDE

struct page* mmap_nopage(
	struct vm_area_struct*	vma,
	unsigned long			address,
	int*					type)
{
	struct page*	page;

	mmap_nopage_common(vma, address, &page);

	if (type)
		type[0]	= VM_FAULT_MINOR;

	return(page);
}

#endif



/******************************************************************************
*
*	Function:	proc_get_info
*
*	Purpose:
*
*		Implement the get_info() service for /proc file system support.
*
*	Arguments:
*
*		page	The data produced is put here.
*
*		start	Records pointer to where the data in "page" begins.
*
*		offset	The offset into the file where we're to begin.
*
*		count	The limit to the amount of data we can produce.
*
*	Returned:
*
*		int		The number of characters written.
*
******************************************************************************/

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26))

int proc_get_info(
	char*	page,
	char**	start,
	off_t	offset,
	int		count)
{
	int	eof;
	int	i;

	i	= proc_read(page, start, offset, count, &eof, NULL);
	return(i);
}

#endif



#endif
