// Application - HPDI32 - Any OS
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/jumpers/jumpers.c $
// $Rev: 23670 $
// $Date: 2013-10-31 16:22:17 -0500 (Thu, 31 Oct 2013) $

#include "main.h"



// variables	***************************************************************

static	U32	_bsr		= 0;
static	int	_supported	= 0;



/*****************************************************************************
*
*	Function:	_bsr_test
*
*	Purpose:
*
*		Report the jumper information directly from the Board Status Register.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _bsr_test(void* handle)
{
	int	errs	= 0;
	U32	field;
	U32	status;
	U32	v0;
	U32	v1;
	U32	v2;
	U32	v3;
	U32	v4;

	label_text("Board Status Register");
	status	= hpdi32_reg_read(handle, HPDI32_BSR, &_bsr);

	if (status == GSC_SUCCESS)
	{
		printf("0x%08lX\n", (long) _bsr);
		label_level_inc();

		label_text("User Jumper 0 (D16)");
		field	= (_bsr & 0x00010000) >> 16;
		printf("0x---%01lX----  ", (long) field);
		printf("%s\n", _supported ? (field ? "On" : "Off") : "Unsupported");

		label_text("User Jumper 1 (D17)");
		field	= (_bsr & 0x00020000) >> 16;
		printf("0x---%01lX----  ", (long) field);
		printf("%s\n", _supported ? (field ? "On" : "Off") : "Unsupported");

		label_level_dec();
	}
	else
	{
		errs	+= label_fail_func(1, "hpdi32_reg_read", status);
	}

	label_level_inc();

	label_text("Macros");
	printf("\n");
	label_level_inc();

	label_text("HPDI32_BSR_BJ0");
	field	= HPDI32_BSR_BJ0;

	if (field == 0x00010000)
	{
		printf("PASS  (0x%lX)\n", (long) HPDI32_BSR_BJ0);
	}
	else
	{
		errs++;
		printf("FAIL <---  (0x%lX)\n", (long) HPDI32_BSR_BJ0);
	}

	label_text("HPDI32_BSR_BJ1");
	field	= HPDI32_BSR_BJ1;

	if (field == 0x00020000)
	{
		printf("PASS  (0x%lX)\n", (long) HPDI32_BSR_BJ1);
	}
	else
	{
		errs++;
		printf("FAIL <---  (0x%lX)\n", (long) HPDI32_BSR_BJ1);
	}

	label_text("HPDI32_BSR_BJ_DECODE");
	field	= HPDI32_BSR_BJ_DECODE(HPDI32_BSR_BJ0 | HPDI32_BSR_BJ1);

	if (field == 3)
	{
		printf("PASS\n");
	}
	else
	{
		errs++;
		printf(	"FAIL <---  (expect 0x%lX, got 0x%lX)\n",
				(long) 0x3,
				(long) field);
	}

	label_text("HPDI32_BSR_BJ_ENCODE");
	v0	= HPDI32_BSR_BJ_ENCODE(0);
	v1	= HPDI32_BSR_BJ_ENCODE(1);
	v2	= HPDI32_BSR_BJ_ENCODE(2);
	v3	= HPDI32_BSR_BJ_ENCODE(3);
	v4	= HPDI32_BSR_BJ_ENCODE(0xFFFFFFFF);

	if (v0 != 0x00000)
	{
		errs++;
		printf(	"FAIL <---  (%d. expect 0x%lX, got 0x%lX)\n",
				__LINE__,
				(long) 0,
				(long) v0);
	}
	else if (v1 != 0x10000)
	{
		errs++;
		printf(	"FAIL <---  (%d. expect 0x%lX, got 0x%lX)\n",
				__LINE__,
				(long) 0x10000,
				(long) v1);
	}
	else if (v2 != 0x20000)
	{
		errs++;
		printf(	"FAIL <---  (%d. expect 0x%lX, got 0x%lX)\n",
				__LINE__,
				(long) 0x20000,
				(long) v2);
	}
	else if (v3 != 0x30000)
	{
		errs++;
		printf(	"FAIL <---  (%d. expect 0x%lX, got 0x%lX)\n",
				__LINE__,
				(long) 0x30000,
				(long) v3);
	}
	else if (v4 != 0x30000)
	{
		errs++;
		printf(	"FAIL <---  (%d. expect 0x%lX, got 0x%lX)\n",
				__LINE__,
				(long) 0x30000,
				(long) v4);
	}
	else
	{
		printf("PASS\n");
	}

	label_level_dec();
	label_level_dec();

	return(errs);
}



/*****************************************************************************
*
*	Function:	_feature_test
*
*	Purpose:
*
*		Test the User Jumper feature.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _feature_test(void* handle)
{
	int				errs	= 0;
	unsigned long	f1;
	unsigned long	f2;
	U32				status;

	label_text("Feature Test");
	printf("\n");

	label_level_inc();

	label_text("HPDI32_MISC_FEATURES_USER_JUMPERS");
	status	= hpdi32_config(handle,
							HPDI32_MISC_FEATURES,
							0,
							HPDI32_MISC_FEATURES_USER_JUMPERS,
							&f1);

	if (status != GSC_SUCCESS)
	{
		_supported	= 0;
		errs		+= label_fail_func(1, "hpdi32_config", status);
	}
	else if (f1 == HPDI32_MISC_FEATURES_PRESENT)
	{
		_supported	= 1;
		printf("PASS  (feature IS supported)\n");
	}
	else if (f1 == HPDI32_MISC_FEATURES_ABSENT)
	{
		_supported	= 1;
		printf("PASS  (feature is NOT supported)\n");
	}
	else
	{
		_supported	= 0;
		errs++;
		printf("FAIL <---  (invalid response: 0x%lX)\n", (long) f1);
	}

	label_text("HPDI32_MISC_FEATURES__USER_JUMPERS");
	status	= HPDI32_MISC_FEATURES__USER_JUMPERS(handle, &f2);

	if (status != GSC_SUCCESS)
	{
		errs	+= label_fail_func(1, "HPDI32_MISC_FEATURES__USER_JUMPERS", status);
	}
	else if (f2 != f1)
	{
		errs++;
		printf("FAIL <---  (expected 0x%lX, got 0x%lX)\n", (long) f1, (long) f2);
	}
	else if (f2 == HPDI32_MISC_FEATURES_PRESENT)
	{
		printf("PASS  (feature IS supported)\n");
	}
	else if (f2 == HPDI32_MISC_FEATURES_ABSENT)
	{
		printf("PASS  (feature is NOT supported)\n");
	}
	else
	{
		errs++;
		printf("FAIL <---  (invalid response: 0x%lX)\n", f1);
	}

	label_level_dec();

	return(errs);
}



/*****************************************************************************
*
*	Function:	_parm_test
*
*	Purpose:
*
*		Test the User Jumper feature.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _parm_test(void* handle)
{
	U32				bsr_value	= (_bsr & 0x30000) >> 16;
	int				errs		= 0;
	unsigned long	jumpers;
	U32				status;

	label_text("Parameter Test");
	printf("\n" );

	label_level_inc();

	label_text("HPDI32_MISC_BOARD_JUMPERS");
	status	= hpdi32_config(handle, HPDI32_MISC_BOARD_JUMPERS, 0, 0, &jumpers);

	if (status != GSC_SUCCESS)
	{
		errs	+= label_fail_func(1, "hpdi32_config", status);
	}
	else if (jumpers != bsr_value)
	{
		errs++;
		printf(	"FAIL <---  (expected 0x%lX, got 0x%lX)\n",
				(long) bsr_value,
				(long) jumpers);
	}
	else if (jumpers & 0xFFFFFFFC)
	{
		errs++;
		printf("FAIL <---  (unexpected value: 0x%lX)\n", (long) jumpers);
	}
	else
	{
		printf("PASS  (0x%lX)\n", (long) jumpers);
	}

	label_text("HPDI32_MISC_BOARD_JUMPERS__GET");
	status	= HPDI32_MISC_BOARD_JUMPERS__GET(handle, &jumpers);

	if (status != GSC_SUCCESS)
	{
		errs	+= label_fail_func(1, "HPDI32_MISC_BOARD_JUMPERS__GET", status);
	}
	else if (jumpers != bsr_value)
	{
		errs++;
		printf(	"FAIL <---  (expected 0x%lX, got 0x%lX)\n",
				(long) bsr_value,
				(long) jumpers);
	}
	else if (jumpers & 0xFFFFFFFC)
	{
		errs++;
		printf("FAIL <---  (unexpected value: 0x%lX)\n", (long) jumpers);
	}
	else
	{
		printf("PASS  (0x%lX)\n", (long) jumpers);
	}

	label_level_dec();

	return(errs);
}



/*****************************************************************************
*
*	Function:	jumper_tests
*
*	Purpose:
*
*		Perform a test of the jumper macros.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int	jumper_tests(void* handle)
{
	int	errs	= 0;

	label_init(38);

	label_text("Jumpers");
	printf("\n");
	label_level_inc();

	errs	+= _feature_test(handle);
	errs	+= _bsr_test(handle);
	errs	+= _parm_test(handle);

	label_level_dec();
	return(errs);
}


