// Application - HPDI32 - Any OS
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/rthrottle/main.c $
// $Rev: 23669 $
// $Date: 2013-10-31 16:21:56 -0500 (Thu, 31 Oct 2013) $

#include "main.h"



// variables ******************************************************************

static	int	_block_qty		= 1024;
static	int	_block_size		= 1024;
static	int	_continuous		= 0;
static	int	_def_index		= 0;
static	int	_ignore_errors	= 0;
static	int	_io_mode		= HPDI32_IO_MODE_PIO;
static	int	_minute_limit	= 0;
static	int	_receive		= 0;
static	int	_rthrottle		= 1;
static	int	_save_data		= 0;
static	int	_test_limit		= 0;
static	int	_transmit		= 0;



//*****************************************************************************
static int _parse_args(int argc, char** argv)
{
	char	buf[64];
	char	c;
	int		errs	= 0;
	int		i;
	int		j;
	int		k;

	printf("rthrottle - Remote Throttle demonstration (version %s)\n", VERSION);
	printf("USAGE: rthrottle [tx|rx] <-c> <-C> <-dma> <-dmdma> <-m#> <-n#> <-pio> <-q#> <-r-> <-s> <-s#> <index>\n");
	printf("  tx|rx   Transmit or receive data (required).\n");
	printf("  -c      Continuous testing, but stop on first error.\n");
	printf("  -C      Continuous testing, disregard errors.\n");
	printf("  -dma    Transfer data using block mode DMA.\n");
	printf("  -dmdma  Transfer data using Demand Mode DMA.\n");
	printf("  -m#     Run for at most # minutes (default is %d). For -c and -C.\n", _minute_limit);
	printf("  -n#     Repeat test at most # times (default is %d). For -c and -C.\n", _test_limit);
	printf("  -pio    Transfer data using PIO. (This is the default mode.)\n");
	printf("  -q#     Block Quantity: Transfer # blocks (default is %d).\n", _block_qty);
	printf("  -r-     Disable remote throttling.\n");
	printf("  -s      Save the data to rx_data.txt or rx_data.txt, respectively.\n");
	printf("  -s#     Tx only: Block Size: Transmit # words per block (1 to 1M, default is %d).\n", _block_size);
	printf("  index   The zero based index of the board to access.\n");
	printf("  NOTE:   Hit Ctrl-C to interrupt continuous testing.\n");

	label_level_inc();

	for (i = 0; i < argc; i++)
	{
		sprintf(buf, "Argument %d", i);
		label_text(buf);
		printf("%s\n", argv[i]);
	}

	label_level_dec();

	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "-c") == 0)
		{
			_continuous		= 1;
			_ignore_errors	= 0;
			continue;
		}

		if (strcmp(argv[i], "-C") == 0)
		{
			_continuous		= 1;
			_ignore_errors	= 1;
			continue;
		}

		if (strcmp(argv[i], "-dma") == 0)
		{
			_io_mode	= HPDI32_IO_MODE_DMA;
			continue;
		}

		if (strcmp(argv[i], "-dmdma") == 0)
		{
			_io_mode	= HPDI32_IO_MODE_DMDMA;
			continue;
		}

		if (strcmp(argv[i], "-pio") == 0)
		{
			_io_mode	= HPDI32_IO_MODE_PIO;
			continue;
		}

		if (strcmp(argv[i], "-r-") == 0)
		{
			_rthrottle	= 0;
			continue;
		}

		if (strcmp(argv[i], "-s") == 0)
		{
			_save_data	= 1;
			continue;
		}

		if (strcmp(argv[i], "rx") == 0)
		{
			if (_transmit)
			{
				errs++;
				printf("ERROR: \"tx\" already specified: %s\n", argv[i]);
				continue;
			}

			_receive	= 1;
			continue;
		}

		if (strcmp(argv[i], "tx") == 0)
		{
			if (_receive)
			{
				errs++;
				printf("ERROR: \"rx\" already specified: %s\n", argv[i]);
				continue;
			}

			_transmit	= 1;
			continue;
		}

		if ((argv[i][0] == '-') && (argv[i][1] == 'm') && (argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%d%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_minute_limit	= k;
				continue;
			}

			errs++;
			printf("ERROR: invalid argument: %s\n", argv[i]);
			continue;
		}

		if ((argv[i][0] == '-') && (argv[i][1] == 'n') && (argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%d%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_test_limit	= k;
				continue;
			}

			errs++;
			printf("ERROR: invalid argument: %s\n", argv[i]);
			continue;
		}

		if ((argv[i][0] == '-') && (argv[i][1] == 'q') && (argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%d%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_block_qty	= k;
				continue;
			}

			errs++;
			printf("ERROR: invalid argument: %s\n", argv[i]);
			continue;
		}

		if ((argv[i][0] == '-') && (argv[i][1] == 's') && (argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%d%c", &k, &c);

			if ((j == 1) && (k > 0) && (k <= _1M))
			{
				_block_size	= k;
				continue;
			}

			errs++;
			printf("ERROR: invalid argument: %s\n", argv[i]);
			continue;
		}

		j	= sscanf(argv[i], "%d%c", &k, &c);

		if ((j == 1) && (k >= 0))
		{
			_def_index	= k;
			continue;
		}

		errs++;
		printf("ERROR: can't access configuration file: '%s'\n", argv[i]);
		break;
	}

	if ((_transmit == 0) && (_receive == 0))
	{
		errs++;
		printf("ERROR: \"tx\" or \"rx\" not specified.\n");
	}

	return(errs);
}



//*****************************************************************************
static int _perform_tests(void* dev, int index)
{
	int			errs;
	char*		psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= asctime(stm);
	label_text("Performing Tests");
	printf("%s", psz);

	errs	= id_hardware(dev, index);

	if (errs)
	{
	}
	else if ((_receive) && (_transmit))
	{
		errs++;
		printf("%d. INTERNAL ERROR: both data directions specified\n", __LINE__);
	}
	else if (_receive)
	{
		errs	= rx_data(dev, _rthrottle, _block_size * 4, _block_qty, _save_data, _io_mode);
	}
	else if (_transmit)
	{
		errs	= tx_data(dev, _rthrottle, _block_size * 4, _block_qty, _save_data, _io_mode);
	}
	else
	{
		errs++;
		printf("%d. INTERNAL ERROR: data direction not specified\n", __LINE__);
	}

	return(errs);
}



//*****************************************************************************
int buffer_alloc(void* dev, unsigned long* api_size, void** api_buf)
{
	int		errs	= 0;
	long	want;

	label_text("Buffer Allocation");
	printf("\n");
	label_level_inc();

	for (;;)	// A convenience loop.
	{
		if (api_size == NULL)
		{
			errs++;
			printf("FAIL <---  (NULL pointer: api_size)\n");
		}

		if (api_buf == NULL)
		{
			errs++;
			printf("FAIL <---  (NULL pointer: api_buf)\n");
		}

		if (errs)
			break;

		want	= api_size[0];
		errs	+= dev_tx_io_buffer_size	(dev, -1, 1, api_size[0], api_size);
		errs	+= dev_tx_io_buffer_pointer	(dev, -1, 1, api_buf);
		label_text("Buffer Type");

		if (api_buf[0] == NULL)
		{
			errs++;
			printf("FAIL <---  (API Buffer allocation failed.)\n");
		}
		else if (want > api_size[0])
		{
			errs++;
			printf(	"FAIL <---  (API Buffer allocation failure:"
					"requested %ld bytes, got %ld bytes)\n",
					(long) want,
					(long) api_size[0]);
		}
		else
		{
			printf("API Buffer\n");
		}

		break;
	}

	label_level_dec();
	return(errs);
}



/*****************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Program entry point.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were seen.
*		EXIT_FAILURE	Errors were seen.
*
*****************************************************************************/

int main(int argc, char** argv)
{
	void*	dev;
	int		errs;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		qty;
	long	secs;
	time_t	test;
	long	tests		= 0;
	time_t	t_limit;

	for (;;)
	{
		errs	= 0;
		label_init(28);
		test	= time(NULL);
		errs	+= _parse_args(argc, argv);

		if (errs)
			break;

		id_host();
		t_limit	= exec + (_minute_limit * 60);
		errs	+= dev_api_status();

		if (errs)
			break;

		qty		= dev_board_count();
		errs	+= (qty < 0) ? 1 : 0;

		if (errs)
			break;

		label_text("Testing Board Index");
		printf("%d\n", _def_index);
		errs	+= dev_open((U8) _def_index, &dev);

		if (errs == 0)
		{
			errs	= _perform_tests(dev, _def_index);
			errs	+= dev_close((U8) _def_index, dev);
		}

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf("\nRESULTS: FAIL <---");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= now - test;
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= now - exec;
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (_continuous == 0)
			break;

		if ((_ignore_errors == 0) && (errs))
			break;

		if ((_minute_limit == 0) && (_test_limit == 0))
			break;

		if ((_minute_limit) && (now >= t_limit))
			break;

		if ((_test_limit) && (tests >= _test_limit))
			break;
	}

	errs	= errs ? EXIT_FAILURE : EXIT_SUCCESS;
	return(errs);
}


