// Application - HPDI32 - Linux
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/rthrottle/rx.c $
// $Rev: 23669 $
// $Date: 2013-10-31 16:21:56 -0500 (Thu, 31 Oct 2013) $

#include "main.h"



//*****************************************************************************
static int _dev_config(void* dev, int rthrottle, int io_mode)
{
	int				errs		= 0;
	unsigned long	cmd_3;

	label_text("Rx Configuration");
	printf("\n");
	label_level_inc();

	if (rthrottle)
		cmd_3		= HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL;
	else
		cmd_3		= HPDI32_CABLE_COMMAND_MODE_GPIO_IN;

	errs	+= dev_init						(dev, -1, 1);
	errs	+= dev_favor_tx					(dev, -1, 1, HPDI32_MISC_FAVOR_TX_DISABLE, NULL);
	errs	+= dev_tx_rx_tri_state			(dev, -1, 1, HPDI32_MISC_TX_RX_TRI_STATE_DISABLE, NULL);
	errs	+= dev_cable_command_mode_0		(dev, -1, 1, HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL, NULL);
	errs	+= dev_cable_command_mode_1		(dev, -1, 1, HPDI32_CABLE_COMMAND_MODE_GPIO_IN, NULL);
	errs	+= dev_cable_command_mode_2		(dev, -1, 1, HPDI32_CABLE_COMMAND_MODE_GPIO_IN, NULL);
	errs	+= dev_cable_command_mode_3		(dev, -1, 1, cmd_3, NULL);
	errs	+= dev_cable_command_mode_4		(dev, -1, 1, HPDI32_CABLE_COMMAND_MODE_GPIO_IN, NULL);
	errs	+= dev_cable_command_mode_5		(dev, -1, 1, HPDI32_CABLE_COMMAND_MODE_GPIO_IN, NULL);
	errs	+= dev_cable_command_mode_6		(dev, -1, 1, HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL, NULL);
	errs	+= dev_rx_fifo_ae_level			(dev, -1, 1, 64, NULL);
	errs	+= dev_rx_fifo_af_level			(dev, -1, 1, 64, NULL);
	errs	+= dev_rx_io_data_size			(dev, -1, 1, HPDI32_IO_DATA_SIZE_32_BITS, NULL);
	errs	+= dev_rx_io_mode				(dev, -1, 1, io_mode, NULL);
	errs	+= dev_rx_io_dma_control_mode	(dev, -1, 1, HPDI32_IO_DMA_CONTROL_MODE_AUTOMATIC, NULL);
	errs	+= dev_rx_io_dma_channel_sel	(dev, -1, 1, HPDI32_IO_DMA_CHANNEL_SEL_STATIC, NULL);
	errs	+= dev_rx_io_dma_priority		(dev, -1, 1, HPDI32_IO_DMA_PRIORITY_ENABLE, NULL);
	errs	+= dev_rx_io_overlap_enable		(dev, -1, 1, HPDI32_IO_OVERLAP_ENABLE_NO, NULL);
	errs	+= dev_rx_io_pio_threshold		(dev, -1, 1, 32, NULL);
	errs	+= dev_rx_io_single_cycle		(dev, -1, 1, HPDI32_IO_SINGLE_CYCLE_PRESENT, NULL);
	errs	+= dev_rx_io_timeout			(dev, -1, 1, 10, NULL);

	label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _wait_for_tx_enabled(void* dev, int seconds)
{
	long	delta;
	int		errs	= 0;
	time_t	now;
	time_t	start;
	U32		status;
	U32		value;

	label_text("Wait for Tx Enabled");
	start	= time(NULL);

	for (;;)
	{
		now	= time(NULL);

		if (now > (start + seconds))
		{
			errs++;
			printf("FAIL <---  (%d second time out expired)\n", seconds);
			break;
		}

		status	= hpdi32_reg_read(dev, HPDI32_BSR, &value);

		if (status != GSC_SUCCESS)
		{
			errs++;
			printf("FAIL <---  (GPIO read status: 0x%lX)\n", (long) status);
			break;
		}

		if (value & 0x20)	// D5 is the Tx Enabled input: hi = enabled
		{
			delta	= now - start;
			printf("Done  (%ld second%s)\n", delta, (delta == 1) ? "" : "s");
			break;
		}

		usleep(100000);
	}

	return(errs);
}



//*****************************************************************************
static int _wait_for_tx_disabled(void* dev, int seconds)
{
	long	delta;
	int		errs	= 0;
	time_t	now;
	time_t	start;
	U32		status;
	U32		value;

	label_text("Wait for Tx Disabled");
	start	= time(NULL);

	for (;;)
	{
		now	= time(NULL);

		if (now > (start + seconds))
		{
			errs++;
			printf("FAIL <---  (%d second time out expired)\n", seconds);
			break;
		}

		status	= hpdi32_reg_read(dev, HPDI32_BSR, &value);

		if (status != GSC_SUCCESS)
		{
			errs++;
			printf("FAIL <---  (GPIO read status: 0x%lX)\n", (long) status);
			break;
		}

		if (value & 0x20)	// D5 is the Tx Enabled input: hi = enabled
		{
			usleep(100000);
			continue;
		}

		delta	= now - start;
		printf("Done  (%ld second%s)\n", delta, (delta == 1) ? "" : "s");
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _rx_data(void* dev, int block_qty, int api_size, void* api_buf, int errs, int save)
{
	long		blocks	= 0;
	char		buf[64];
	const char*	factor	= "";
	FILE*		file	= NULL;
	U32			get		= (U32) api_size;
	U32			got;
	const char*	name	= "rx_data.txt";
	U32			status;
	long long	total	= 0;

	label_text("Receiving Data");

	if (errs)
	{
		errs	= 0;
		printf("Aborted due to setup errors.\n");
	}
	else
	{
		printf("Working ... ");
		label_level_inc();

		if (save)
		{
			file	= file_create(name);

			if (file)
				printf("(data being saved) ... ");
			else
				printf("Data not being saved (file access error).\n");
		}

		for (blocks = 0; block_qty > 0; block_qty--, blocks++)
		{
			if ((blocks % 50) == 0)
			{
				printf("\n");
				sprintf(buf, "Block #%ld", (long) blocks + 1);
				label_text(buf);
			}

			printf(".");
			fflush(stdout);
			memset(api_buf, 'a', get);
			status	= hpdi32_read(dev, api_buf, get, &got);
			total	+= got;

			if (status != GSC_SUCCESS)
			{
				errs++;
				printf("FAIL <---  (hpdi32_read(), status 0x%lX)\n", (long) status);
				break;
			}

			if (got != get)
			{
				errs++;
				printf("FAIL <---  (hpdi32_read(), got too few bytes: %ld)\n", (long) got);
				break;
			}

			if (file)
				file_write(name, file, api_buf, got);
		}

		if (file)
			file_close(name, file);

		if (errs == 0)
			printf("\n");

		label_text("Results");

		if ((total) && ((total % 1024) == 0))
		{
			total	/= 1024;
			factor	= "K";
		}

		if ((total) && ((total % 1024) == 0))
		{
			total	/= 1024;
			factor	= "M";
		}

		if ((total) && ((total % 1024) == 0))
		{
			total	/= 1024;
			factor	= "G";
		}

		if ((total) && ((total % 1024) == 0))
		{
			total	/= 1024;
			factor	= "T";
		}

		label_long_comma(total);
		printf("%s byte%s (1K = 1024)\n", factor, (total == 1) ? "" : "s");
		errs	+= dev_rx_overrun(dev, -1, 1, HPDI32_RX_OVERRUN_IGNORE, NULL);
		label_level_dec();
	}

	return(errs);
}



//*****************************************************************************
int rx_data(void* dev, int rthrottle, int block_size, int block_qty, int save, int io_mode)
{
	void*			api_buf		= NULL;
	unsigned long	api_size	= block_size;
	int				errs		= 0;

	for (;;)	// A convenience loop.
	{
		label_text("Setup");
		printf("\n");
		label_level_inc();
		errs	+= _dev_config(dev, rthrottle, io_mode);
		errs	+= buffer_alloc(dev, &api_size, &api_buf);

		// Tx is enabled before Rx is enabled.
		errs	+= _wait_for_tx_enabled(dev, 30);
		errs	+= dev_rx_enable(dev, -1, 1, HPDI32_RX_ENABLE_YES, NULL);
		label_level_dec();

		errs	+= _rx_data(dev, block_qty, block_size, api_buf, errs, save);

		label_text("Cleanup");
		printf("\n");
		label_level_inc();

		// Tx is disabled before Rx is disabled.
		errs	+= _wait_for_tx_disabled(dev, 30);
		errs	+= dev_rx_enable(dev, -1, 1, HPDI32_RX_ENABLE_NO, NULL);

		errs	+= dev_rx_io_buffer_size(dev, -1, 1, 0, NULL);
		label_level_dec();
		break;
	}

	return(errs);
}


